#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Deflip", Category = "Spreads", Help = "Can be used to flip angles in a circle and checks for minimum value change", Tags = "angle, curvature")]
	#endregion PluginInfo
	public class SpreadsDeflipNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Input", DefaultValue = 1.0)]
		IDiffSpread<double> FInput;

		[Input("Threshold", DefaultValue = 0.25)]
		IDiffSpread<double> FThresh;

		[Input("Flip Value", DefaultValue = 0.5)]
		IDiffSpread<double> FFlip;

		[Output("Output")]
		ISpread<double> FOutput;
		
		[Output("Is Flipped")]
		ISpread<bool> FIsFlippedOutput;

		[Import()]
		ILogger FLogger;
		#endregion fields & pins

		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			FOutput.SliceCount = SpreadMax;
			FIsFlippedOutput.SliceCount = SpreadMax;

			if (FInput.IsChanged || FThresh.IsChanged || FFlip.IsChanged) 
			{
				var flip = false;
				FOutput[0] = FInput[0]; //leave first one
				FIsFlippedOutput[0] = false;
				var last = FInput[0];
				
				for (int i = 1; i < SpreadMax; i++)
				{
					if (Math.Abs(FInput[i] - FInput[i - 1]) > FThresh[0]) 
						flip = !flip;
					
					var flipValue = FFlip[0];
					
					//calc new value
					var curent = flip ? FInput[i] + flipValue : FInput[i];
					
					//minimum value change
					var diff = last - curent;
					if(diff > flipValue) curent += 2*flipValue;
					else if(diff < -flipValue) curent -= 2*flipValue;
					
					//assign
					FOutput[i] = last = curent;
					FIsFlippedOutput[i] = flip;
				}
			}
		}
	}
}
